package banco.fachada;

import banco.cliente.Cliente;
import banco.cliente.CadastroClientes;
import banco.cliente.ListaClientes;
import banco.cliente.ClienteExistenteException;
import banco.cliente.ClienteInexistenteException;
import banco.conta.Conta;
import banco.conta.ContaExistenteException;
import banco.conta.ContaInexistenteException;
import banco.conta.SaldoInsuficienteException;
import banco.conta.ArrayContas;
import banco.conta.ColecaoContasIteravel;
import banco.conta.CadastroContas;

public class Fachada {

  private static Fachada instancia;
  private CadastroContas contas;
  private CadastroClientes clientes;

  protected Fachada() {
	  initCadastros();
  }

  private void initCadastros() {
	ArrayContas rep = new ArrayContas();
	contas = new CadastroContas(rep);
	ListaClientes repClientes = new ListaClientes();
	clientes = new CadastroClientes(repClientes);
  }


  public static Fachada obterInstancia() {
	 if (instancia == null) {
	   instancia = new Fachada();
	 }
	 return instancia;
  }

  //metodos par amanipular clientes
  public void atualizarCliente(Cliente c) throws ClienteInexistenteException{
	  clientes.atualizar(c);
  }
  public Cliente procurarCliente(String cpf) throws ClienteInexistenteException{
	return clientes.procurar(cpf);
  }
  public void inserirCliente(Cliente c) throws ClienteExistenteException{
	clientes.inserir(c);
  }
  public void removerCliente(String cpf) throws ClienteInexistenteException{
	  clientes.remover(cpf);
  }

  //metodos para manipular contas
  public void atualizarConta(Conta c) throws ContaInexistenteException{
	contas.atualizar(c);
  }
  public Conta procurarConta(String n) throws ContaInexistenteException{
	  return contas.procurar(n);
  }
  public void inserirConta(Conta c)
    throws ContaExistenteException, ClienteInexistenteException{

    Cliente cli = c.getCliente();
    if(cli != null){
      clientes.procurar(cli.getCpf());
      contas.inserir(c);
    }else {
      throw new IllegalArgumentException("cliente nulo");
    }
  }
  public void removerConta(String n) throws ContaInexistenteException{
	  contas.remover(n);
  }
  public void creditar(String n, double v) throws ContaInexistenteException{
    contas.creditar(n, v);
  }
  public void debitar(String n, double v) throws ContaInexistenteException,
    SaldoInsuficienteException{
    contas.debitar(n, v);
  }
  public void transferir(String origem, String destino, double val)
    throws ContaInexistenteException, SaldoInsuficienteException{
	  contas.transferir(origem, destino, val);
  }
  public ColecaoContasIteravel procurarContas(String cpfCliente){
    return contas.procurarContas(cpfCliente);
  }
}
